@@ -137,6 +137,7 @@ struct GSLockPlugPrivate
 
         guint        idle_id;
         guint        password_check_idle_id;
+        guint        password_reset_idle_id;
         guint        response_idle_id;
 
         GTimeVal     start_time;
@@ -276,6 +277,42 @@ set_dialog_sensitive (GSLockPlug *plug,
         gtk_widget_set_sensitive (plug->action_area, sensitive);
 }
 
+static void
+remove_monitor_idle (GSLockPlug *plug)
+{
+        if (plug->priv->idle_id > 0) {
+                g_source_remove (plug->priv->idle_id);
+                plug->priv->idle_id = 0;
+        }
+}
+
+static void
+remove_password_check_idle (GSLockPlug *plug)
+{
+        if (plug->priv->password_check_idle_id > 0) {
+                g_source_remove (plug->priv->password_check_idle_id);
+                plug->priv->password_check_idle_id = 0;
+        }
+}
+
+static void
+remove_password_reset_idle (GSLockPlug *plug)
+{
+        if (plug->priv->password_reset_idle_id > 0) {
+                g_source_remove (plug->priv->password_reset_idle_id);
+                plug->priv->password_reset_idle_id = 0;
+        }
+}
+
+static void
+remove_response_idle (GSLockPlug *plug)
+{
+        if (plug->priv->response_idle_id > 0) {
+                g_source_remove (plug->priv->response_idle_id);
+                plug->priv->response_idle_id = 0;
+        }
+}
+
 static void
 gs_lock_plug_response (GSLockPlug *plug,
                        gint        response_id)
@@ -292,20 +329,10 @@ gs_lock_plug_response (GSLockPlug *plug,
                 return;
         }
 
-        if (plug->priv->idle_id > 0) {
-                g_source_remove (plug->priv->idle_id);
-                plug->priv->idle_id = 0;
-        }
-
-        if (plug->priv->password_check_idle_id > 0) {
-                g_source_remove (plug->priv->password_check_idle_id);
-                plug->priv->password_check_idle_id = 0;
-        }
-
-        if (plug->priv->response_idle_id > 0) {
-                g_source_remove (plug->priv->response_idle_id);
-                plug->priv->response_idle_id = 0;
-        }
+        remove_monitor_idle (plug);
+        remove_password_check_idle (plug);
+        remove_password_reset_idle (plug);
+        remove_response_idle (plug);
 
         if (response_id == GS_LOCK_PLUG_RESPONSE_CANCEL) {
                 gtk_entry_set_text (GTK_ENTRY (plug->priv->password_entry), "");
@@ -314,7 +341,7 @@ gs_lock_plug_response (GSLockPlug *plug,
         if (response_id == GS_LOCK_PLUG_RESPONSE_OK) {
                 gint current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (plug->priv->notebook));
 
-                if (current_page == 0) {
+                if (current_page == AUTH_PAGE) {
                         set_dialog_sensitive (plug, FALSE);
                         set_status_text (plug, _("Checking password..."));
 
@@ -350,7 +377,6 @@ monitor_progress (GSLockPlug *plug)
         GTimeVal now;
         glong    elapsed;
         glong    remaining;
-        char    *message;
 
         g_get_current_time (&now);
 
@@ -359,15 +385,17 @@ monitor_progress (GSLockPlug *plug)
         remaining = plug->priv->timeout - elapsed;
 
         if ((remaining <= 0) || (remaining > plug->priv->timeout)) {
-                message = g_strdup (_("Time has expired."));
                 set_dialog_sensitive (plug, FALSE);
-                set_status_text (plug, message);
-                g_free (message);
+                set_status_text (plug, _("Time has expired."));
 
-                if (plug->priv->response_idle_id == 0)
-                        plug->priv->response_idle_id = g_timeout_add (2000,
-                                                                      (GSourceFunc)response_idle_cb,
-                                                                      plug);
+                if (plug->priv->response_idle_id != 0)
+                        g_warning ("Response idle ID already set but shouldn't be");
+
+                remove_response_idle (plug);
+
+                plug->priv->response_idle_id = g_timeout_add (2000,
+                                                              (GSourceFunc)response_idle_cb,
+                                                              plug);
                 return FALSE;
         }
         return TRUE;
@@ -407,10 +435,7 @@ is_capslock_on (void)
 static void
 restart_monitor_progress (GSLockPlug *plug)
 {
-        if (plug->priv->idle_id > 0) {
-                g_source_remove (plug->priv->idle_id);
-                plug->priv->idle_id = 0;
-        }
+        remove_monitor_idle (plug);
 
         g_get_current_time (&plug->priv->start_time);
         plug->priv->idle_id = g_timeout_add (50,
@@ -447,15 +472,22 @@ static void
 gs_lock_plug_size_request (GtkWidget      *widget,
                            GtkRequisition *requisition)
 {
+        int mod_width;
+        int mod_height;
+
         if (GTK_WIDGET_CLASS (parent_class)->size_request)
                 GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
 
-        /* Force width to be at least 1.2 times the height.
-         * This ensures enough space for the text entry and
-         * makes for a more pleasing aspect ratio */
-        if (requisition->width < requisition->height) {
-                requisition->width = requisition->height * 1.2;
+        mod_width = requisition->height * 1.3;
+        mod_height = requisition->width / 1.6;
+        if (requisition->width < mod_width) {
+                /* if the dialog is tall fill out the width */
+                requisition->width = mod_width;
+        } else if (requisition->height < mod_height) {
+                /* if the dialog is wide fill out the height */
+                requisition->height = mod_height;
         }
+
 }
 
 static void
@@ -613,6 +645,20 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
                                                                G_PARAM_READWRITE));
 }
 
+static gboolean
+password_reset_idle_cb (GSLockPlug *plug)
+{
+        gtk_entry_set_text (GTK_ENTRY (plug->priv->password_entry), "");
+        set_status_text (plug, "");
+        set_dialog_sensitive (plug, TRUE);
+
+        restart_monitor_progress (plug);
+
+        gtk_widget_grab_focus (plug->priv->password_entry);
+
+        return FALSE;
+}
+
 static gboolean
 password_check_idle_cb (GSLockPlug *plug)
 {
@@ -636,10 +682,11 @@ password_check_idle_cb (GSLockPlug *plug)
                                0,
                                GS_LOCK_PLUG_RESPONSE_OK);
         } else {
-                if (plug->priv->response_idle_id == 0)
-                        plug->priv->response_idle_id = g_timeout_add (4000,
-                                                                      (GSourceFunc)response_idle_cb,
-                                                                      plug);
+                remove_password_reset_idle (plug);
+
+                plug->priv->password_reset_idle_id = g_timeout_add (3000,
+                                                                    (GSourceFunc)password_reset_idle_cb,
+                                                                    plug);
 
                 set_status_text (plug, _("That password was incorrect."));
 
@@ -979,7 +1026,7 @@ user_displays_changed_cb (FusaUser           *user,
         name = fusa_user_get_user_name (user);
         n_displays = fusa_user_get_n_displays (user);
         is_active = n_displays > 0;
-        pixbuf = fusa_user_render_icon (user, data->tree, icon_size, is_active);
+        pixbuf = fusa_user_render_icon (user, data->tree, icon_size);
         label = get_user_display_label (user);
 
         filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (data->tree));
@@ -1049,7 +1096,7 @@ populate_model (GSLockPlug   *plug,
                 n_displays = fusa_user_get_n_displays (user);
                 is_active = n_displays > 0;
 
-                pixbuf = fusa_user_render_icon (user, plug->priv->user_treeview, icon_size, is_active);
+                pixbuf = fusa_user_render_icon (user, plug->priv->user_treeview, icon_size);
 
                 label = get_user_display_label (user);
 
@@ -1257,29 +1304,6 @@ get_user_display_name (void)
         return name;
 }
 
-static void
-label_set_big_bold (GtkLabel *label)
-{
-        PangoAttrList        *pattrlist;
-        PangoAttribute       *attr;
-
-        pattrlist = pango_attr_list_new ();
-
-        attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-        attr->start_index = 0;
-        attr->end_index = G_MAXINT;
-        pango_attr_list_insert (pattrlist, attr);
-
-        attr = pango_attr_scale_new (1.2);
-        attr->start_index = 0;
-        attr->end_index = G_MAXINT;
-        pango_attr_list_insert (pattrlist, attr);
-
-        gtk_label_set_attributes (label, pattrlist);
-
-        pango_attr_list_unref (pattrlist);
-}
-
 static gboolean
 check_user_file (const gchar *filename,
 		 uid_t        user,
@@ -1343,6 +1367,8 @@ get_face_image ()
                                                            NULL);
         }
 
+        g_free (path);
+
         if (! pixbuf)
                 return NULL;
 
@@ -1372,21 +1398,13 @@ create_page_one (GSLockPlug *plug)
         vbox = gtk_vbox_new (FALSE, 12);
         gtk_container_add (GTK_CONTAINER (align), vbox);
 
-        /* should we make this a gconf preference? */
-        if (0) {
-                /* translators: %s is a computer hostname */
-                str = g_strdup_printf (_("Welcome to %s"), g_get_host_name ());
-                widget = gtk_label_new (str);
-                gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
-                gtk_misc_set_alignment (GTK_MISC (widget), 0.5, 0.5);
-                g_free (str);
-
-                label_set_big_bold (GTK_LABEL (widget));
-        }
-
         widget = get_face_image ();
-        if (widget)
-                gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
+        if (! widget) {
+                /* placeholder */
+                widget = gtk_label_new (NULL);
+        }
+        gtk_box_pack_start (GTK_BOX (vbox), widget, TRUE, TRUE, 0);
+        gtk_misc_set_alignment (GTK_MISC (widget), 0.5, 1.0);
 
         vbox2 = gtk_vbox_new (FALSE, 0);
         gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
@@ -1406,7 +1424,7 @@ create_page_one (GSLockPlug *plug)
         gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->username_label, FALSE, FALSE, 0);
 
         vbox2 = gtk_vbox_new (FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (vbox), vbox2, TRUE, TRUE, 0);
 
         hbox = gtk_hbox_new (FALSE, 6);
         gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);
@@ -1436,6 +1454,35 @@ create_page_one (GSLockPlug *plug)
         profile_end ("end", "page one");
 }
 
+static void
+constrain_list_size (GtkWidget      *widget,
+                     GtkRequisition *requisition,
+                     GSLockPlug     *plug)
+{
+        GtkRequisition req;
+        int            max_height;
+        int            page;
+
+        /* don't do anything if we are on the auth page */
+        page = gtk_notebook_get_current_page (GTK_NOTEBOOK (plug->priv->notebook));
+        if (page == AUTH_PAGE)
+                return;
+
+        /* constrain height to be the tree height up to a max */
+        max_height = (gdk_screen_get_height (gtk_widget_get_screen (widget))) / 4;
+        gtk_widget_size_request (plug->priv->user_treeview, &req);
+
+        requisition->height = MIN (req.height, max_height);
+}
+
+static void
+setup_list_size_constraint (GtkWidget  *widget,
+                            GSLockPlug *plug)
+{
+        g_signal_connect (widget, "size-request",
+                          G_CALLBACK (constrain_list_size), plug);
+}
+
 static void
 create_page_two (GSLockPlug *plug)
 {
@@ -1460,6 +1507,8 @@ create_page_two (GSLockPlug *plug)
                                         GTK_POLICY_AUTOMATIC);
         gtk_box_pack_start (GTK_BOX (vbox), userlist_scroller, TRUE, TRUE, 0);
 
+        setup_list_size_constraint (userlist_scroller, plug);
+
         plug->priv->user_treeview = gtk_tree_view_new ();
         gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (plug->priv->user_treeview), FALSE);
         gtk_container_add (GTK_CONTAINER (userlist_scroller), plug->priv->user_treeview);
@@ -1547,7 +1596,7 @@ gs_lock_plug_init (GSLockPlug *plug)
         plug->priv->notebook = gtk_notebook_new ();
         gtk_notebook_set_show_tabs (GTK_NOTEBOOK (plug->priv->notebook), FALSE);
         gtk_notebook_set_show_border (GTK_NOTEBOOK (plug->priv->notebook), FALSE);
-        gtk_box_pack_start (GTK_BOX (plug->vbox), plug->priv->notebook, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (plug->vbox), plug->priv->notebook, TRUE, TRUE, 0);
 
         /* Page 1 */
 
@@ -1604,20 +1653,10 @@ gs_lock_plug_finalize (GObject *object)
         if (plug->priv->fusa_manager)
                 g_object_unref (plug->priv->fusa_manager);
 
-        if (plug->priv->password_check_idle_id > 0) {
-                g_source_remove (plug->priv->password_check_idle_id);
-                plug->priv->password_check_idle_id = 0;
-        }
-
-        if (plug->priv->response_idle_id > 0) {
-                g_source_remove (plug->priv->response_idle_id);
-                plug->priv->response_idle_id = 0;
-        }
-
-        if (plug->priv->idle_id > 0) {
-                g_source_remove (plug->priv->idle_id);
-                plug->priv->idle_id = 0;
-        }
+        remove_password_check_idle (plug);
+        remove_password_reset_idle (plug);
+        remove_response_idle (plug);
+        remove_monitor_idle (plug);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
