@@ -31,8 +31,8 @@
 #include "gs-window.h"
 #include "gs-job.h"
 #include "gs-grab.h"
-
 #include "gs-fade.h"
+#include "gs-debug.h"
 
 static void gs_manager_class_init (GSManagerClass *klass);
 static void gs_manager_init       (GSManager      *manager);
@@ -137,16 +137,19 @@ gs_manager_set_throttle_enabled (GSManager *manager,
 
                 manager->priv->throttle_enabled = throttle_enabled;
 
-                for (l = manager->priv->jobs; l; l = l->next) {
-                        if (throttle_enabled) {
-                                gs_job_stop (GS_JOB (l->data));
-                        } else {
-                                gs_job_start (GS_JOB (l->data));
-                        }
-                }
+                if (! manager->priv->dialog_up) {
 
-                for (l = manager->priv->windows; l; l = l->next) {
-                        gs_window_clear (l->data);
+                        for (l = manager->priv->jobs; l; l = l->next) {
+                                if (throttle_enabled) {
+                                        gs_job_stop (GS_JOB (l->data));
+                                } else {
+                                        gs_job_start (GS_JOB (l->data));
+                                }
+                        }
+
+                        for (l = manager->priv->windows; l; l = l->next) {
+                                gs_window_clear (l->data);
+                        }
                 }
         }
 }
@@ -161,8 +164,9 @@ gs_manager_set_lock_active (GSManager *manager,
                 GSList *l;
 
                 manager->priv->lock_active = lock_active;
-                for (l = manager->priv->windows; l; l = l->next)
+                for (l = manager->priv->windows; l; l = l->next) {
                         gs_window_set_lock_enabled (l->data, lock_active);
+                }
         }
 }
 
@@ -187,8 +191,9 @@ gs_manager_set_logout_enabled (GSManager *manager,
                 GSList *l;
 
                 manager->priv->logout_enabled = logout_enabled;
-                for (l = manager->priv->windows; l; l = l->next)
+                for (l = manager->priv->windows; l; l = l->next) {
                         gs_window_set_logout_enabled (l->data, logout_enabled);
+                }
         }
 }
 
@@ -202,8 +207,9 @@ gs_manager_set_user_switch_enabled (GSManager *manager,
                 GSList *l;
 
                 manager->priv->user_switch_enabled = user_switch_enabled;
-                for (l = manager->priv->windows; l; l = l->next)
+                for (l = manager->priv->windows; l; l = l->next) {
                         gs_window_set_user_switch_enabled (l->data, user_switch_enabled);
+                }
         }
 }
 
@@ -273,8 +279,9 @@ gs_manager_set_logout_timeout (GSManager *manager,
                 GSList *l;
 
                 manager->priv->logout_timeout = logout_timeout;
-                for (l = manager->priv->windows; l; l = l->next)
+                for (l = manager->priv->windows; l; l = l->next) {
                         gs_window_set_logout_timeout (l->data, logout_timeout);
+                }
         }
 }
 
@@ -301,11 +308,7 @@ select_theme (GSManager *manager)
         g_return_val_if_fail (manager != NULL, NULL);
         g_return_val_if_fail (GS_IS_MANAGER (manager), NULL);
 
-        if (manager->priv->saver_mode == GS_MODE_BLANK_ONLY)
-                return NULL;
-
-        if (manager->priv->saver_mode == GS_MODE_DONT_BLANK) {
-                g_warning ("Selecting no screensaver theme since screensaver is disabled.");
+        if (manager->priv->saver_mode == GS_MODE_BLANK_ONLY) {
                 return NULL;
         }
 
@@ -332,14 +335,17 @@ gs_manager_cycle (GSManager *manager)
         g_return_val_if_fail (manager != NULL, FALSE);
         g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
 
-        if (! manager->priv->active)
+        if (! manager->priv->active) {
                 return FALSE;
+        }
 
-        if (manager->priv->dialog_up)
+        if (manager->priv->dialog_up) {
                 return FALSE;
+        }
 
-        if (manager->priv->throttle_enabled)
+        if (manager->priv->throttle_enabled) {
                 return FALSE;
+        }
 
         theme = select_theme (manager);
 
@@ -348,8 +354,9 @@ gs_manager_cycle (GSManager *manager)
                 gs_job_set_theme (GS_JOB (l->data), theme, NULL);
 
                 /* this success flag is kinda bogus */
-                if (gs_job_start (GS_JOB (l->data)))
+                if (gs_job_start (GS_JOB (l->data))) {
                         success = TRUE;
+                }
         }
 
         return success;
@@ -361,8 +368,9 @@ cycle_timeout (GSManager *manager)
         g_return_val_if_fail (manager != NULL, FALSE);
         g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
 
-        if (! manager->priv->dialog_up)
+        if (! manager->priv->dialog_up) {
                 gs_manager_cycle (manager);
+        }
 
         return TRUE;
 }
@@ -687,6 +695,8 @@ window_dialog_up_cb (GSWindow  *window,
         g_return_if_fail (manager != NULL);
         g_return_if_fail (GS_IS_MANAGER (manager));
 
+        gs_debug ("Handling dialog up");
+
         manager->priv->dialog_up = TRUE;
 
         /* Grab keyboard so dialog can be used */
@@ -696,11 +706,14 @@ window_dialog_up_cb (GSWindow  *window,
 
         /* Make all other windows insensitive so we don't get events */
         for (l = manager->priv->windows; l; l = l->next) {
-                if (l->data != window)
+                if (l->data != window) {
                         gtk_widget_set_sensitive (GTK_WIDGET (l->data), FALSE);
+                }
         }
 
         if (! manager->priv->throttle_enabled) {
+                gs_debug ("Suspending jobs");
+
                 for (l = manager->priv->jobs; l; l = l->next) {
                         gs_job_suspend (GS_JOB (l->data), TRUE);
                 }
@@ -723,7 +736,11 @@ window_dialog_down_cb (GSWindow  *window,
 
         if (! manager->priv->throttle_enabled) {
                 for (l = manager->priv->jobs; l; l = l->next) {
-                        gs_job_suspend (GS_JOB (l->data), FALSE);
+                        if (gs_job_is_running (GS_JOB (l->data))) {
+                                gs_job_suspend (GS_JOB (l->data), FALSE);
+                        } else {
+                                gs_job_start (GS_JOB (l->data));
+                        }
                 }
         }
 
@@ -890,8 +907,9 @@ gs_manager_create (GSManager *manager)
         }
         g_slist_free (manager->priv->windows);
 
-        for (i = 0; i < n_screens; i++)
+        for (i = 0; i < n_screens; i++) {
                 gs_manager_create_window (manager, gdk_display_get_screen (display, i));
+        }
 }
 
 GSManager *
@@ -941,6 +959,8 @@ gs_manager_activate (GSManager *manager)
         if (! manager->priv->windows)
                 gs_manager_create (GS_MANAGER (manager));
 
+        manager->priv->active = TRUE;
+
         /* fade to black and show windows */
         do_fade = TRUE;
         if (do_fade) {
@@ -957,8 +977,6 @@ gs_manager_activate (GSManager *manager)
                 show_windows (manager->priv->windows);
         }
 
-        manager->priv->active = TRUE;
-
         return TRUE;
 }
 
@@ -1006,16 +1024,17 @@ gs_manager_set_active (GSManager *manager,
 {
         gboolean res;
 
-        if (active)
+        if (active) {
                 res = gs_manager_activate (manager);
-        else
+        } else {
                 res = gs_manager_deactivate (manager);
+        }
 
         return res;
 }
 
 gboolean
-gs_manager_is_active (GSManager *manager)
+gs_manager_get_active (GSManager *manager)
 {
         g_return_val_if_fail (manager != NULL, FALSE);
         g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
@@ -1034,8 +1053,9 @@ gs_manager_request_unlock (GSManager *manager)
         g_return_if_fail (manager != NULL);
         g_return_if_fail (GS_IS_MANAGER (manager));
 
-        if (! manager->priv->active)
+        if (! manager->priv->active) {
                 return;
+        }
 
         /* Find the screen that contains the pointer */
         display = gdk_display_get_default ();
