@@ -33,6 +33,7 @@
 
 #include "gs-window.h"
 #include "subprocs.h"
+#include "gs-debug.h"
 
 static void gs_window_class_init (GSWindowClass *klass);
 static void gs_window_init       (GSWindow      *window);
@@ -132,8 +133,9 @@ set_invisible_cursor (GdkWindow *window,
 
         gdk_window_set_cursor (window, cursor);
 
-        if (cursor)
+        if (cursor) {
                 gdk_cursor_unref (cursor);
+        }
 }
 
 /* derived from tomboy */
@@ -159,17 +161,78 @@ gs_window_override_user_time (GSWindow *window)
         gdk_x11_window_set_user_time (GTK_WIDGET (window)->window, ev_time);
 }
 
+static void
+clear_children (Window window)
+{
+        Window            root;
+        Window            parent;
+        Window           *children;
+        unsigned int      n_children;
+        int               status;
+
+        children = NULL;
+        status = XQueryTree (GDK_DISPLAY (), window, &root, &parent, &children, &n_children);
+
+        if (status == 0) {
+                if (children)
+                        XFree (children);
+                return;
+        }
+
+        if (children) {
+                while (n_children) {
+                        Window child;
+
+                        child = children [--n_children];
+
+                        XClearWindow (GDK_DISPLAY (), child);
+                        clear_children (child);
+                }
+
+                XFree (children);
+        }
+}
+
+static void
+clear_all_children (GSWindow *window)
+{
+        GdkWindow *w;
+
+        gs_debug ("Clearing all child windows");
+
+        gdk_error_trap_push ();
+
+        w = GTK_WIDGET (window)->window;
+
+        clear_children (GDK_WINDOW_XID (w));
+
+        gdk_display_sync (gtk_widget_get_display (GTK_WIDGET (window)));
+        gdk_error_trap_pop ();
+}
+
 void
 gs_window_clear (GSWindow *window)
 {
-        GdkColor     color = { 0, 0, 0 };
+        GdkColor     color = { 0, 0x0000, 0x0000, 0x0000 };
         GdkColormap *colormap;
+        GtkStateType state;
+
+        gs_debug ("Clearing window");
+
+        state = (GtkStateType) 0;
+        while (state < (GtkStateType) G_N_ELEMENTS (GTK_WIDGET (window)->style->bg)) {
+                gtk_widget_modify_bg (GTK_WIDGET (window), state, &color);
+                state++;
+        }
 
-        gtk_widget_modify_bg (GTK_WIDGET (window), GTK_STATE_NORMAL, &color);
         colormap = gdk_drawable_get_colormap (GTK_WIDGET (window)->window);
         gdk_colormap_alloc_color (colormap, &color, FALSE, TRUE);
         gdk_window_set_background (GTK_WIDGET (window)->window, &color);
         gdk_window_clear (GTK_WIDGET (window)->window);
+
+        /* If a screensaver theme adds child windows we need to clear them too */
+        clear_all_children (window);
+
         gdk_flush ();
 }
 
@@ -207,20 +270,21 @@ gs_window_move_resize_window (GSWindow *window,
 
         g_assert (GTK_WIDGET_REALIZED (widget));
 
-        if (move && resize)
+        if (move && resize) {
                 gdk_window_move_resize (widget->window,
                                         window->priv->geometry.x,
                                         window->priv->geometry.y,
                                         window->priv->geometry.width,
                                         window->priv->geometry.height);
-        else if (move)
+        } else if (move) {
                 gdk_window_move (widget->window,
                                  window->priv->geometry.x,
                                  window->priv->geometry.y);
-        else if (resize)
+        } else if (resize) {
                 gdk_window_resize (widget->window,
                                    window->priv->geometry.width,
                                    window->priv->geometry.height);
+        }
 }
 
 static void
@@ -230,18 +294,19 @@ gs_window_real_unrealize (GtkWidget *widget)
                                               screen_size_changed,
                                               widget);
 
-        if (GTK_WIDGET_CLASS (parent_class)->unrealize)
+        if (GTK_WIDGET_CLASS (parent_class)->unrealize) {
                 GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+        }
 }
 
 static void
 gs_window_real_realize (GtkWidget *widget)
 {
-        if (GTK_WIDGET_CLASS (parent_class)->realize)
+        if (GTK_WIDGET_CLASS (parent_class)->realize) {
                 GTK_WIDGET_CLASS (parent_class)->realize (widget);
+        }
 
         gs_window_override_user_time (GS_WINDOW (widget));
-        gs_window_clear (GS_WINDOW (widget));
 
         gs_window_move_resize_window (GS_WINDOW (widget), TRUE, TRUE);
 
@@ -330,11 +395,29 @@ gs_window_raise (GSWindow *window)
 
         g_return_if_fail (GS_IS_WINDOW (window));
 
+        gs_debug ("Raising screensaver window");
+
         win = GTK_WIDGET (window)->window;
 
         gdk_window_raise (win);
 }
 
+static gboolean
+x11_window_is_ours (Window window)
+{
+        GdkWindow *gwindow;
+        gboolean   ret;
+
+        ret = FALSE;
+
+        gwindow = gdk_window_lookup (window);
+        if (gwindow && (window != GDK_ROOT_WINDOW ())) {
+                ret = TRUE;
+        }
+
+        return ret;
+}
+
 static void
 gs_window_xevent (GSWindow  *window,
                   GdkXEvent *xevent)
@@ -347,11 +430,29 @@ gs_window_xevent (GSWindow  *window,
            ConfigureNofify is used to tell us when windows are raised. */
         switch (ev->xany.type) {
         case MapNotify:
-                gs_window_raise (window);
-                break;
+                {
+                        XMapEvent *xme = &ev->xmap;
+
+                        if (! x11_window_is_ours (xme->window)) {
+                                gs_window_raise (window);
+                        } else {
+                                gs_debug ("not raising our windows");
+                        }
+
+                        break;
+                }
         case ConfigureNotify:
-                gs_window_raise (window);
-                break;
+                {
+                        XConfigureEvent *xce = &ev->xconfigure;
+
+                        if (! x11_window_is_ours (xce->window)) {
+                                gs_window_raise (window);
+                        } else {
+                                gs_debug ("not raising our windows");
+                        }
+
+                        break;
+                }
         default:
                 break;
         }
@@ -385,14 +486,18 @@ gs_window_real_show (GtkWidget *widget)
 {
         GSWindow *window;
 
-        if (GTK_WIDGET_CLASS (parent_class)->show)
+        if (GTK_WIDGET_CLASS (parent_class)->show) {
                 GTK_WIDGET_CLASS (parent_class)->show (widget);
+        }
+
+        gs_window_clear (GS_WINDOW (widget));
 
         set_invisible_cursor (widget->window, TRUE);
 
         window = GS_WINDOW (widget);
-        if (window->priv->timer)
+        if (window->priv->timer) {
                 g_timer_destroy (window->priv->timer);
+        }
         window->priv->timer = g_timer_new ();
 
         remove_watchdog_timer (window);
@@ -421,8 +526,9 @@ gs_window_real_hide (GtkWidget *widget)
 
         remove_watchdog_timer (window);
 
-        if (GTK_WIDGET_CLASS (parent_class)->hide)
+        if (GTK_WIDGET_CLASS (parent_class)->hide) {
                 GTK_WIDGET_CLASS (parent_class)->hide (widget);
+        }
 }
 
 void
@@ -441,6 +547,47 @@ gs_window_get_gdk_window (GSWindow *window)
         return GTK_WIDGET (window)->window;
 }
 
+static GPtrArray *
+get_env_vars (GtkWidget *widget)
+{
+        GPtrArray *env;
+        char      *str;
+        int        i;
+        static const char *allowed_env_vars [] = {
+                "PATH",
+                "SESSION_MANAGER",
+                "XAUTHORITY",
+                "XAUTHLOCALHOSTNAME",
+                "LANG",
+                "LANGUAGE"
+        };
+
+        env = g_ptr_array_new ();
+
+        str = gdk_screen_make_display_name (gtk_widget_get_screen (widget));
+        g_ptr_array_add (env, g_strdup_printf ("DISPLAY=%s", str));
+        g_free (str);
+
+        g_ptr_array_add (env, g_strdup_printf ("HOME=%s",
+                                               g_get_home_dir ()));
+
+        for (i = 0; i < G_N_ELEMENTS (allowed_env_vars); i++) {
+                const char *var;
+                const char *val;
+                var = allowed_env_vars [i];
+                val = g_getenv (var);
+                if (val != NULL) {
+                        g_ptr_array_add (env, g_strdup_printf ("%s=%s",
+                                                               var,
+                                                               val));
+                }
+        }
+
+        g_ptr_array_add (env, NULL);
+
+        return env;
+}
+
 static gboolean
 spawn_on_window (GSWindow *window,
                  char     *command,
@@ -451,7 +598,6 @@ spawn_on_window (GSWindow *window,
 {
         int         argc;
         char      **argv;
-        char       *str;
         GPtrArray  *env;
         GError     *error;
         gboolean    result;
@@ -468,37 +614,11 @@ spawn_on_window (GSWindow *window,
                 return FALSE;
         }
 
-        env = g_ptr_array_new ();
-
-        str = gdk_screen_make_display_name (GTK_WINDOW (window)->screen);
-        g_ptr_array_add (env, g_strdup_printf ("DISPLAY=%s", str));
-        g_free (str);
-
-        g_ptr_array_add (env, g_strdup_printf ("HOME=%s",
-                                               g_get_home_dir ()));
-        if (g_getenv ("PATH"))
-                g_ptr_array_add (env, g_strdup_printf ("PATH=%s",
-                                                       g_getenv ("PATH")));
-        if (g_getenv ("SESSION_MANAGER"))
-                g_ptr_array_add (env, g_strdup_printf ("SESSION_MANAGER=%s",
-                                                       g_getenv ("SESSION_MANAGER")));
-        if (g_getenv ("XAUTHORITY"))
-                g_ptr_array_add (env, g_strdup_printf ("XAUTHORITY=%s",
-                                                       g_getenv ("XAUTHORITY")));
-        if (g_getenv ("XAUTHLOCALHOSTNAME"))
-                g_ptr_array_add (env, g_strdup_printf ("XAUTHLOCALHOSTNAME=%s",
-                                                       g_getenv ("XAUTHLOCALHOSTNAME")));
-        if (g_getenv ("LANG"))
-                g_ptr_array_add (env, g_strdup_printf ("LANG=%s",
-                                                       g_getenv ("LANG")));
-        if (g_getenv ("LANGUAGE"))
-                g_ptr_array_add (env, g_strdup_printf ("LANGUAGE=%s",
-                                                       g_getenv ("LANGUAGE")));
-        g_ptr_array_add (env, NULL);
+        env = get_env_vars (GTK_WIDGET (window));
 
         error = NULL;
         result = gdk_spawn_on_screen_with_pipes (GTK_WINDOW (window)->screen,
-                                                 g_get_home_dir (),
+                                                 NULL,
                                                  argv,
                                                  (char **)env->pdata,
                                                  G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
@@ -510,8 +630,9 @@ spawn_on_window (GSWindow *window,
                                                  NULL,
                                                  &error);
 
-        for (i = 0; i < env->len; i++)
+        for (i = 0; i < env->len; i++) {
                 g_free (g_ptr_array_index (env, i));
+        }
         g_ptr_array_free (env, TRUE);
 
         if (! result) {
@@ -521,10 +642,11 @@ spawn_on_window (GSWindow *window,
                 return FALSE;
         }
 
-        if (pid)
+        if (pid) {
                 *pid = child_pid;
-        else
+        } else {
                 g_spawn_close_pid (child_pid);
+        }
 
         channel = g_io_channel_unix_new (standard_output);
         g_io_channel_set_close_on_unref (channel, TRUE);
@@ -535,8 +657,9 @@ spawn_on_window (GSWindow *window,
                              G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
                              watch_func,
                              user_data);
-        if (watch_id)
+        if (watch_id) {
                 *watch_id = id;
+        }
 
         g_io_channel_unref (channel);
 
@@ -647,12 +770,13 @@ wait_on_child (int pid)
 
  wait_again:
         if (waitpid (pid, &status, 0) < 0) {
-                if (errno == EINTR)
+                if (errno == EINTR) {
                         goto wait_again;
-                else if (errno == ECHILD)
+                } else if (errno == ECHILD) {
                         ; /* do nothing, child already reaped */
-                else
+                } else {
                         g_warning ("waitpid () should not fail in 'GSWindow'");
+                }
         }
 
         return status;
@@ -663,6 +787,8 @@ gs_window_dialog_finish (GSWindow *window)
 {
         g_return_if_fail (GS_IS_WINDOW (window));
 
+        gs_debug ("Dialog finished");
+
         if (window->priv->pid > 0) {
                 int exit_status;
                         
@@ -693,16 +819,18 @@ shake_dialog (GSWindow *window)
                         right = 30;
                 }
 
-                if (! window->priv->box)
+                if (! window->priv->box) {
                         break;
+                }
 
                 gtk_alignment_set_padding (GTK_ALIGNMENT (window->priv->box),
                                            0, 0,
                                            left,
                                            right);
 
-                while (gtk_events_pending ())
+                while (gtk_events_pending ()) {
                         gtk_main_iteration ();
+                }
 
                 g_usleep (10000);
         }
@@ -726,7 +854,7 @@ command_watch (GIOChannel   *source,
 
                 switch (status) {
                 case G_IO_STATUS_NORMAL:
-                        /*g_message ("LINE: %s", line);*/
+                        gs_debug ("command output: %s", line);
 
                         if (strstr (line, "WINDOW ID=")) {
                                 guint32 id;
@@ -792,16 +920,19 @@ is_logout_enabled (GSWindow *window)
 {
         double elapsed;
 
-        if (! window->priv->logout_enabled)
+        if (! window->priv->logout_enabled) {
                 return FALSE;
+        }
 
-        if (! window->priv->logout_command)
+        if (! window->priv->logout_command) {
                 return FALSE;
+        }
 
         elapsed = g_timer_elapsed (window->priv->timer, NULL);
 
-        if (window->priv->logout_timeout < (elapsed * 1000))
+        if (window->priv->logout_timeout < (elapsed * 1000)) {
                 return TRUE;
+        }
 
         return FALSE;
 }
@@ -819,6 +950,8 @@ popup_dialog_idle (GSWindow *window)
         char     *tmp;
         GString  *command;
 
+        gs_debug ("Popping up dialog");
+
         tmp = g_build_filename (LIBEXECDIR, "gnome-screensaver-dialog", NULL);
         command = g_string_new (tmp);
         g_free (tmp);
@@ -841,8 +974,9 @@ popup_dialog_idle (GSWindow *window)
                                   (GIOFunc)command_watch,
                                   window,
                                   &window->priv->watch_id);
-        if (! result)
+        if (! result) {
                 g_warning ("Could not start command: %s", command->str);
+        }
 
         g_string_free (command, TRUE);
 
@@ -856,8 +990,11 @@ gs_window_request_unlock (GSWindow *window)
 {
         g_return_if_fail (GS_IS_WINDOW (window));
 
-        if (window->priv->watch_id)
+        gs_debug ("Requesting unlock");
+
+        if (window->priv->watch_id > 0) {
                 return;
+        }
 
         if (! window->priv->lock_enabled) {
                 add_emit_deactivated_idle (window);
@@ -878,8 +1015,9 @@ gs_window_set_lock_enabled (GSWindow *window,
 {
         g_return_if_fail (GS_IS_WINDOW (window));
 
-        if (window->priv->lock_enabled == lock_enabled)
+        if (window->priv->lock_enabled == lock_enabled) {
                 return;
+        }
 
         window->priv->lock_enabled = lock_enabled;
         g_object_notify (G_OBJECT (window), "lock-enabled");
@@ -928,10 +1066,11 @@ gs_window_set_logout_timeout (GSWindow *window,
 {
         g_return_if_fail (GS_IS_WINDOW (window));
 
-        if (logout_timeout < 0)
+        if (logout_timeout < 0) {
                 window->priv->logout_timeout = 0;
-        else
+        } else {
                 window->priv->logout_timeout = logout_timeout;
+        }
 }
 
 void
@@ -955,8 +1094,9 @@ gs_window_set_monitor (GSWindow *window,
 {
         g_return_if_fail (GS_IS_WINDOW (window));
 
-        if (window->priv->monitor == monitor)
+        if (window->priv->monitor == monitor) {
                 return;
+        }
 
         window->priv->monitor = monitor;
 
@@ -1088,8 +1228,9 @@ gs_window_real_key_press_event (GtkWidget   *widget,
                 queue_key_event (GS_WINDOW (widget), event);
         }
 
-        if (GTK_WIDGET_CLASS (parent_class)->key_press_event)
+        if (GTK_WIDGET_CLASS (parent_class)->key_press_event) {
                 GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
+        }
 
         return TRUE;
 }
@@ -1142,8 +1283,9 @@ gs_window_real_size_request (GtkWidget      *widget,
         window = GS_WINDOW (widget);
         bin = GTK_BIN (widget);
 
-        if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
+        if (bin->child && GTK_WIDGET_VISIBLE (bin->child)) {
                 gtk_widget_size_request (bin->child, requisition);
+        }
 
 	old_geometry = window->priv->geometry;
 
@@ -1152,16 +1294,19 @@ gs_window_real_size_request (GtkWidget      *widget,
         requisition->width  = window->priv->geometry.width;
         requisition->height = window->priv->geometry.height;
 
-        if (! GTK_WIDGET_REALIZED (widget))
+        if (! GTK_WIDGET_REALIZED (widget)) {
                 return;
+        }
 
         if (old_geometry.width  != window->priv->geometry.width ||
-            old_geometry.height != window->priv->geometry.height)
+            old_geometry.height != window->priv->geometry.height) {
                 size_changed = TRUE;
+        }
 
         if (old_geometry.x != window->priv->geometry.x ||
-            old_geometry.y != window->priv->geometry.y)
+            old_geometry.y != window->priv->geometry.y) {
                 position_changed = TRUE;
+        }
 
         gs_window_move_resize_window (window, position_changed, size_changed);
 }
@@ -1323,8 +1468,9 @@ gs_window_finalize (GObject *object)
         remove_request_unlock_idle (window);
         remove_popup_dialog_idle (window);
 
-        if (window->priv->timer)
+        if (window->priv->timer) {
                 g_timer_destroy (window->priv->timer);
+        }
 
         remove_key_events (window);
 
